use rand::{Rng, thread_rng};
use std::sync::{Arc, Mutex};
use std::thread;
use std::os::raw::c_char;
use std::ffi::CString;


// Funkcja wrappera dla wywołania z Pythona
#[no_mangle]
pub extern "C" fn generate_passwords_with_strength_wrapper(
    length: usize,
    num_passwords: usize,
) -> *mut *mut std::os::raw::c_char {
    let passwords = generate_passwords_with_strength_internal(length, num_passwords);

    // Tworzenie wektora wskaźników na C-strings
    let mut password_pointers: Vec<*mut std::os::raw::c_char> = Vec::with_capacity(num_passwords);
    for password in &passwords {
        let c_str = CString::new(password.as_str()).expect("Failed to create C-string");
        let password_ptr = Box::into_raw(c_str.into_boxed_c_str()) as *mut std::os::raw::c_char;
        password_pointers.push(password_ptr);
    }

    password_pointers.as_mut_ptr()
}

// Funkcja generująca hasło o zadanej długości
fn generate_passwords_with_length(length: usize) -> String {
    let chars: Vec<char> = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789!@#$%^&*()".chars().collect();
    let mut rng = thread_rng();
    (0..length)
        .map(|_| chars[rng.gen_range(0..chars.len())])
        .collect()
}

fn check_password_strength_internal(password: &String) -> bool {
    /*
    Funkcja sprawdzająca siłę hasła. Silne hasło będzie spełniać przynajmniej 4 warunki:
    - ma co najmniej 8 znaków
    - ma co najmniej jedną wielką literę
    - ma co najmniej jedną małą literę
    - ma co najmniej jedną cyfrę
    - ma co najmniej jedną znak specjalny
     */
    let mut score = 0;

    // Sprawdzenie długości hasła
    if password.len() >= 8 {
        score += 1;
    }

    // Sprawdzenie występowania małych liter
    if password.chars().any(|c| c.is_ascii_lowercase()) {
        score += 1;
    }

    // Sprawdzenie występowania wielkich liter
    if password.chars().any(|c| c.is_ascii_uppercase()) {
        score += 1;
    }

    // Sprawdzenie występowania cyfr
    if password.chars().any(|c| c.is_ascii_digit()) {
        score += 1;
    }

    // Sprawdzenie występowania znaków specjalnych
    if password.chars().any(|c| c.is_ascii_punctuation()) {
        score += 1;
    }

    // Zwrócenie true, jeśli wynik oceny siły hasła jest większy lub równy 3
    score >= 4
}

fn generate_passwords_with_strength_internal(strength: usize, num_passwords: usize) -> Vec<String> {
    /*
    Funkcja generująca hasła o zadanej długości i liczbie.
    Każde hasło jest generowane w osobnym wątku. Jeżeli hasło spełnia wymagania, wątek kończy działanie.
    Jeżeli wątek nie potrafi wygenerować mocnego hasła w 1000 iteracjach, przerywa działanie.
     */
    let passwords: Arc<Mutex<Vec<String>>> = Arc::new(Mutex::new(Vec::new()));
    let mut handles = vec![];

    for _ in 0..num_passwords {
        let passwords_ref = Arc::clone(&passwords);

        let handle = thread::spawn(move || {
            let mut attempts = 0;
            loop {
                let password = generate_passwords_with_length(strength);
                if check_password_strength_internal(&password) {
                    passwords_ref.lock().unwrap().push(password);
                    break;
                }
                // Jeżeli nie można wygenerować hasła, przerwij działanie
                attempts += 1;
                if attempts >= 1000 {
                    eprintln!("Błąd: Nie można wygenerować hasła spełniającego warunek.");
                    break;
                }
            }
        });

        handles.push(handle);
    }

    for handle in handles {
        handle.join().unwrap();
    }

    Arc::try_unwrap(passwords).unwrap().into_inner().unwrap()
}

// Funkcja zwalniająca zaalokowaną pamięć dla tablicy hasła
#[no_mangle]
pub extern "C" fn free_passwords_array(passwords: *mut *mut c_char, length: usize) {
    unsafe {
        for i in 0..length {
            if !passwords.is_null() {
                let password_ptr = *passwords.offset(i as isize);
                if !password_ptr.is_null() {
                    let _ = CString::from_raw(password_ptr);
                }
            }
        }
    }
}